#include "matrix4x4.h"
#include "vector.h"
#include <string.h> // memcpy

//! Allocates memory. Values are not initialized.
Matrix4x4::Matrix4x4()
	: Matrix3x3(4)
{
}

//! Copy constructor.
//! Allocates memory and copies the matrix contents.
Matrix4x4::Matrix4x4(const Matrix4x4 &m)
	: Matrix3x3(m)
{
}

void Matrix4x4::setTranslation(const Vector &translation)
{
	(Vector&)operator()(0, 3) = translation;
}

Vector Matrix4x4::getTranslation()
{
	return Matrix3x3::getCol(3);
}

// Rotation will be applied before translation
void Matrix4x4::setRigidBodyTransformation(double yaw, double pitch, double roll, const Vector &translation)
{
	setRotation(yaw, pitch, roll);
	setTranslation(translation);
	setFourthRow();
}

// Rotation will be applied before translation
void Matrix4x4::setRigidBodyTransformation(const Quaternion &q, const Vector &translation)
{
	setRotation(q);
	setTranslation(translation);
	setFourthRow();
}

void Matrix4x4::setPerspectiveProjection(double cotanHalfVerticalAoV, double aspectRatio, double zNear, double zFar)
{
	double depth = zNear - zFar;
	operator()(0, 0) = cotanHalfVerticalAoV / aspectRatio;
	operator()(1, 1) = cotanHalfVerticalAoV;
	operator()(2, 2) = (zFar + zNear) / depth;
	operator()(2, 3) = (2. * zFar * zNear) / depth;
	operator()(3, 2) = -1.;
	operator()(3, 3) = 0.;
}

Matrix4x4 Matrix4x4::operator* (const Matrix4x4 &m) const
{
	MatrixNxN res = MatrixNxN::operator* (m);
	return *(Matrix4x4*)&res; // TODO: could be done better
}

Vector Matrix4x4::operator* (const Vector &v) const
{
	Vector res = Matrix3x3::operator*(v);
	for( int i = 0; i < 3; i++ )
		res[i] += (*this)(i, 3);
	return Vector(res);
}

//TODO: same code as in MatrixNxN => is this useful? avoidable?
// allows to use Matrix4x4::operator* instead of MatrixNxN::operator*
Matrix4x4& Matrix4x4::operator*= (const Matrix4x4 &m)
{
	if( size != m.size || !val )
		return *this;

	Matrix4x4 res = *this * (m);
	memcpy(val, res.val, size * size * sizeof(double));
	return *this;
}

void Matrix4x4::setFourthRow(double x, double y, double z, double w)
{
	operator()(3, 0) = x;
	operator()(3, 1) = y;
	operator()(3, 2) = z;
	operator()(3, 3) = w;
}

Matrix4x4& Matrix4x4::invertRotation()
{
	Matrix3x3::transpose();
	return *this;
}

Matrix4x4 & Matrix4x4::invertRigidBodyTransformation()
{
	invertRotation(); // transpose3x3
	setTranslation( Matrix3x3::operator*( -getTranslation() ) );

	return *this;
}
