#ifndef MATRIX4X4_H
#define MATRIX4X4_H
#include "matrix3x3.h"

//! 4*4 matrix for transformations
class Matrix4x4 : public Matrix3x3
{
public:
	//! Allocates memory. Values are not initialized.
	Matrix4x4();

	Matrix4x4( const Matrix4x4 &m );

	//! Matrix multiplication: this = this * m
	//! Should be faster than implementation in MatrixNxN.
	Matrix4x4& operator*= (const Matrix4x4 &m);
	Matrix4x4 operator* (const Matrix4x4 &m) const;

	Vector operator* (const Vector &v) const;

	Vector getTranslation();

	void setTranslation(const Vector &translation);

	void setRigidBodyTransformation(const Quaternion &q, const Vector &translation);
	void setRigidBodyTransformation(double yaw, double pitch, double roll, const Vector &translation);
	void setPerspectiveProjection(double cotanHalfVerticalAoV, double aspectRatio, double zNear, double zFar);

	void setFourthRow(double x = 0., double y = 0., double z = 0., double w = 1.);

	Matrix4x4& invertRotation();

	Matrix4x4 & invertRigidBodyTransformation();
};

#endif // _MATRIX4X4_H
