#include "matrixnxn.h"
#include <string.h> // memcpy

//! Allocates memory if size is specified.
//! Values are not initialized.
MatrixNxN::MatrixNxN( unsigned long size )
	: size(size), val(NULL)
{
	if( size > 0 )
		val = new double[size*size];
}

//! Copy constructor.
//! Allocates memory and copies the matrix contents.
MatrixNxN::MatrixNxN(const MatrixNxN &m)
{
	size = m.size;
	val = NULL;
	if( size > 0 )
	{
		val = new double[size*size];
		memcpy(val, m.val, size * size * sizeof(double));
	}
}

//! Destructor. Frees memory.
MatrixNxN::~MatrixNxN()
{
	if(val)
		delete[] val;
}

void MatrixNxN::setNull()
{
	if(val)
		memset(val, 0, size * size * sizeof(double));
}

void MatrixNxN::setIdentity()
{
	if(val)
	{
		setNull();
		for( int i = 0; i < size; i++ )
			val[ i * (size + 1) ] = 1.;
	}
}

MatrixNxN & MatrixNxN::operator=( const MatrixNxN &m)
{
	if(this->val)
		delete[] val;
	size = m.size;
	val = NULL;
	if( size > 0 )
	{
		val = new double[size*size];
		memcpy(val, m.val, size * size * sizeof(double));
	}
	return *this;
}

//untested
MatrixNxN MatrixNxN::operator* (const MatrixNxN &m) const
{
	if( size != m.size || !val )
		return MatrixNxN();

	MatrixNxN res(size);
	res.setNull();
	for( int i = 0; i < size; i++ )
		for( int j = 0; j < size; j++ )
			for( int k = 0; k < size; k++ )
				res(i, j) += (*this)(i, k) * m(k, j);
	return MatrixNxN(res);
}

MatrixNxN& MatrixNxN::operator*= (const MatrixNxN &m)
{
	if( size != m.size || !val )
		return *this;

	MatrixNxN res = *this * (m);
	memcpy(val, res.val, size * size * sizeof(double));
	return *this;
}
