#ifndef MATRIXNXN_H
#define MATRIXNXN_H

//! N*N matrix (Square matrix)
//! Values are stored in column-major layout (for OpenGL).
//! Access through operator() is row-major.
class MatrixNxN
{
public:
	double *val;
	unsigned long size; // Full matrix is size*size

	//! Allocates memory if size is specified.
	//! Values are not initialized.
	MatrixNxN( unsigned long size = 0 );

	//! Takes ownership of the pointer.
	//! It will be freed on deletion.
	MatrixNxN( double *val, unsigned long size );

	//! Copy constructor.
	//! Allocates memory and copies the matrix contents.
	MatrixNxN(const MatrixNxN &m);

	//! Destructor. Frees memory.
	~MatrixNxN();

	MatrixNxN& operator= (const MatrixNxN &m);
	inline double& operator() (unsigned long row, unsigned long col)
	{
		return val[col * size + row];
	}

	inline double& getCol(unsigned long col)
	{
		return val[col * size];
	}

	inline double operator() (unsigned long row, unsigned long col) const
	{
		return val[col * size + row];
	}

	MatrixNxN operator* (const MatrixNxN &m) const;

	//! Matrix multiplication: this = this * m
	//! Implementation in Matrix3x3 and Matrix4x4 should be faster.
	virtual MatrixNxN& operator*= (const MatrixNxN &m);

	//! Initializes identity matrix.
	//! Memory should have been already allocated.
	void setIdentity();

	//! Clears matrix (all 0).
	void setNull();
};

#endif // _MATRIXNXN_H
