#ifndef VECTOR_H
#define VECTOR_H

class Angle;

//! 3D vector or point, in cartesian or spherical coordinates
class Vector
{
public:
	//! A vertex (or point), can be represented by a vector structure
	typedef Vector Vertex;
	union
	{
		// Vector definition in cartesian coordinates
		struct
		{
			double x, y, z;
		};

		// Vector definition in spherical coordinates
		struct
		{
			double radius, latitude, longitude;
		};

		double coord[3]; // A vector is a set of 3 coordinates
		double val[3]; // To keep consistent with Matrix definition
	};
	Vector(double x = 0., double y = 0., double z = 0.);

	//! Build a vector from 2 points
	Vector(const Vertex &a, const Vertex &b);
	Vector(const float *a, const float *b);

	Vector operator-() const;

	Vector operator+(const Vector &v) const;
	Vector operator-(const Vector &v) const;
	Vector& operator+=(const Vector &v);
	Vector& operator-= (const Vector &v);

	Vector operator*(double mult) const;
	Vector operator/(double div) const;
	Vector& operator*=(double mult);
	Vector& operator/=(const double div);

	double operator*(const Vector &v)const; // scalar product (dot product)
	Vector operator/(const Vector &v) const; // cross product

	double operator[](unsigned int index) const;
	double& operator[](unsigned int index);

	double sqNorm() const;
	double norm() const;
	Vector& normalize();
	Vector normalized() const;

	Vector cartesianToSpherical() const;
	Vector sphericalToCartesian() const;
	static Vector sphericalToCartesian( double radius, const Angle &latitude, const Angle &longitude );
};

//! 2D vector or texture coordinates
class Vector2
{
public:
	union
	{
		struct
		{
			double x, y;
		};
		struct
		{
			double u, v;
		};
		double coord[2];
	};
	Vector2(double x = 0., double y = 0.);
};
#endif // VECTOR_H
